package tech.mercantec.easyeat.helpers

import android.content.Context
import android.util.Log
import android.widget.Toast
import kotlinx.serialization.Serializable
import kotlinx.serialization.SerializationException
import tech.mercantec.easyeat.BuildConfig
import java.net.HttpURLConnection
import java.net.URL
import kotlinx.serialization.json.*
import kotlinx.serialization.serializer
import java.io.IOException

class ApiRequestException(message: String, cause: Throwable?) : Exception(message, cause)

class HttpResponse(val body: String, val code: Int)

fun request(method: String, path: String, data: String?): HttpResponse {
    val url = URL(BuildConfig.API_BASE_URL + path)

    try {
        with(url.openConnection() as HttpURLConnection) {
            requestMethod = method

            if (data != null) {
                setRequestProperty("Content-Type", "application/json")

                outputStream.write(data.toByteArray())
                outputStream.flush()
            }

            if (responseCode >= 400)
                return HttpResponse(errorStream.readBytes().decodeToString(), responseCode)

            return HttpResponse(inputStream.readBytes().decodeToString(), responseCode)
        }
    } catch (e: IOException) {
        Log.e("EasyEat", e.toString())

        throw ApiRequestException("Failed to connect to server: " + e.message, e)
    }
}

@Serializable
class HttpErrorResponse(val message: String)

inline fun <reified Req, reified Res> requestJson(method: String, path: String, data: Req?): Res {
    val requestJson =
        if (data != null)
            Json.encodeToString(serializer<Req>(), data)
        else null

    val response = request(method, path, requestJson)

    if (response.code >= 400) {
        try {
            val error = Json.decodeFromString<HttpErrorResponse>(response.body)

            throw ApiRequestException(error.message, null)
        } catch (e: SerializationException) {
            if (e.message != null)
                Log.e("EasyEat", e.message!!)
            Log.e("EasyEat", response.body)

            throw ApiRequestException("Request failed with HTTP status code ${response.code}", e)
        }
    }

    try {
        return Json.decodeFromString<Res>(response.body)
    } catch (e: SerializationException) {
        if (e.message != null)
            Log.e("EasyEat", e.message!!)
        Log.e("EasyEat", response.body)

        throw ApiRequestException("Failed to parse response: $response", e)
    }
}
